from datetime import timedelta

from pyreindexer import RxConnector
from pyreindexer.exceptions import ApiError
from pyreindexer.query import CondType


def create_index_example(db, namespace):
    index_definition = {
        'name': 'id',
        'json_paths': ['id'],
        'field_type': 'int',
        'index_type': 'hash',
        'is_pk': True,
        'is_array': False,
        'is_dense': False,
        'is_sparse': False,
        'collate_mode': 'none',
        'sort_order_letters': '',
        'expire_after': 0,
        'config': {},
    }

    try:
        db.index_add(namespace, index_definition)
    except ApiError:
        db.index_drop(namespace, 'id', timedelta(milliseconds = 1000))
        db.index_add(namespace, index_definition)


def update_index_example(db, namespace):
    index_definition_modified = {
        'name': 'id',
        'json_paths': ['id'],
        'field_type': 'int64',
        'index_type': 'hash',
        'is_pk': True,
        'is_array': False,
        'is_dense': True,
        'is_sparse': False,
        'collate_mode': 'none',
        'sort_order_letters': '',
        'expire_after': 0,
        'config': {},
    }
    db.index_update(namespace, index_definition_modified)


def create_items_example(db, namespace):
    items_count = 10

    for i in range(0, items_count):
        item = {'id': i + 1, 'name': 'item_' + str(i % 2), 'value': 'check'}
        db.item_upsert(namespace, item)


def select_item_query_example(db, namespace):
    item_name_for_lookup = 'item_0'

    return db.select(f"SELECT * FROM {namespace} WHERE name='{item_name_for_lookup}'", timedelta(milliseconds = 1000))

def select_all_item_query_example(db, namespace):
    return db.select(f"SELECT * FROM {namespace}", timedelta(milliseconds = 1000))

def print_all_records_from_namespace(db, namespace, message):
    selected_items_tr = select_all_item_query_example(db, namespace)

    res_count = selected_items_tr.count()
    print(message, res_count)

    for item in selected_items_tr:
        print('Item: ', item)

def transaction_example(db, namespace, items_in_base):
    # start transaction
    transaction = db.new_transaction(namespace)

    items_count = len(items_in_base)

    # delete first few items
    for i in range(int(items_count / 2)):
        transaction.delete(items_in_base[i])

    # update last one item, overwrite field 'value'
    item = items_in_base[items_count - 1]
    item['value'] = 'transaction was here'
    transaction.update(item)

    # stop transaction and commit changes to namespace
    transaction.commit(timedelta(milliseconds = 1000))

    print_all_records_from_namespace(db, namespace, 'Transaction results count: ')

def query_example(db, namespace):
    # query all items
    any_items = (db.new_query(namespace)
                 .where('value', CondType.CondAny)
                 .sort('id')
                 .execute())
    print(f'Query results count (Any): {any_items.count()}')
    for item in any_items:
        print('Item: ', item)

    # query some items
    selected_items = (db.new_query(namespace)
                      .where('value', CondType.CondEq, 'check')
                      .sort('id')
                      .limit(4)
                      .execute(timedelta(milliseconds = 1000)))
    print(f'Query results count (limited): {selected_items.count()}')
    for item in selected_items:
        print('Item: ', item)

    # delete some items
    del_count = (db.new_query(namespace)
                 .where('name', CondType.CondEq, 'item_1')
                 .delete(timedelta(milliseconds = 1000)))
    print(f'Deleted count: {del_count}')

    # query all actual items
    any_items = (db.new_query(namespace)
                 .where('value', CondType.CondAny)
                 .must_execute())
    print(f'Query results count (Any after delete): {any_items.count()}')
    for item in any_items:
        print('Item: ', item)

def rx_example():
    db = RxConnector('builtin:///tmp/pyrx', max_replication_updates_size = 10 * 1024 * 1024)
    #    db = RxConnector('cproto://127.0.0.1:6534/pyrx', enable_compression = True, fetch_amount = 500)

    namespace = 'test_table'

    db.namespace_open(namespace)

    create_index_example(db, namespace)
    update_index_example(db, namespace)

    create_items_example(db, namespace)

    selected_items = select_item_query_example(db, namespace)

    res_count = selected_items.count()
    print('Results count: ', res_count)

    # disposable QueryResults iterator
    items_copy = []
    for item in selected_items:
        items_copy.append(item)
        print('Item: ', item)

    # won't be iterated again
    for item in selected_items:
        print('Item: ', item)

    transaction_example(db, namespace, items_copy)

    query_example(db, namespace)

    db.close()


if __name__ == "__main__":
    rx_example()



from datetime import timedelta

from hamcrest import *

from pyreindexer.exceptions import ApiError


class TestSqlQueries:
    def test_sql_select(self, db, namespace, index, item):
        # Given("Create namespace with item")
        # When ("Execute SQL query SELECT")
        query = f"SELECT * FROM {namespace}"
        items_list = list(db.query.sql(query, timeout=timedelta(milliseconds=1000)))
        # Then ("Check that selected item is in result")
        assert_that(items_list, equal_to([item]), "Can't SQL select data")

    def test_sql_select_with_join(self, db, namespace, index, items, second_namespace, second_item):
        # Given("Create two namespaces")
        # When ("Execute SQL query SELECT with JOIN")
        query = f"SELECT id FROM {namespace} INNER JOIN {second_namespace} " \
                f"ON {namespace}.id = {second_namespace}.id"
        item_list = list(db.query.sql(query))
        # Then ("Check that selected item is in result")
        item_with_joined = {'id': 1, f'joined_{second_namespace}': [second_item]}
        assert_that(item_list, equal_to([item_with_joined]),
                    "Can't SQL select data with JOIN")

    def test_sql_select_with_condition(self, db, namespace, index, items):
        # Given("Create namespace with item")
        # When ("Execute SQL query SELECT")
        item_id = 3
        query = f"SELECT * FROM {namespace} WHERE id = {item_id}"
        items_list = list(db.query.sql(query))
        # Then ("Check that selected item is in result")
        assert_that(items_list, equal_to([items[item_id]]), "Can't SQL select data with condition")

    def test_sql_update(self, db, namespace, index, item):
        # Given("Create namespace with item")
        # When ("Execute SQL query UPDATE")
        item["val"] = "new_val"
        query = f"UPDATE {namespace} SET \"val\" = '{item['val']}' WHERE id = 100"
        items_list = list(db.query.sql(query))
        # Then ("Check that item is updated")
        assert_that(items_list, equal_to([item]), "Can't SQL update data")

    def test_sql_delete(self, db, namespace, index, item):
        # Given("Create namespace with item")
        # When ("Execute SQL query DELETE")
        query_delete = f"DELETE FROM {namespace} WHERE id = 100"
        db.query.sql(query_delete)
        # Then ("Check that item is deleted")
        query_select = f"SELECT * FROM {namespace}"
        item_list = list(db.query.sql(query_select))
        assert_that(item_list, empty(), "Can't SQL delete data")

    def test_sql_select_with_syntax_error(self, db, namespace, index, item):
        # Given("Create namespace with item")
        # When ("Execute SQL query SELECT with incorrect syntax")
        query = "SELECT *"
        # Then ("Check that selected item is in result")
        assert_that(calling(db.query.sql).with_args(query),
                    raises(ApiError, pattern="Expected .* but found"),
                    "Error wasn't raised when syntax was incorrect")

    def test_sql_select_with_aggregations(self, db, namespace, index, items):
        # Given("Create namespace with item")
        # When ("Insert items into namespace")
        select_result = db.query.sql(f"SELECT min(id), max(id), avg(id), sum(id) FROM {namespace}").get_agg_results()
        assert_that(select_result, has_length(4), "The aggregation result does not contain all aggregations")

        ids = [i["id"] for i in items]
        expected_values = {"min": min(ids), "max": max(ids), "avg": sum(ids) / len(items), "sum": sum(ids)}

        # Then ("Check that returned agg results are correct")
        for agg in select_result:
            assert_that(agg['value'], equal_to(expected_values[agg['type']]),
                        f"Incorrect aggregation result for {agg['type']}")

    def test_sql_select_timeout_small(self, db, namespace, index):
        # Given("Create namespace with items")
        items = [{"id": i, "val": f"testval{i}"} for i in range(10000)]
        for item in items:
            db.item.insert("new_ns", item)
        # When ("Try to execute SQL query SELECT with small timeout")
        query = ("SELECT * FROM new_ns WHERE id > -1 MERGE (SELECT * FROM new_ns WHERE val < 'testval1000') MERGE "
                 "(SELECT * FROM new_ns WHERE val > 'testval1000' AND id RANGE(1,9000))")
        assert_that(calling(db.query.sql).with_args(query, timeout=timedelta(milliseconds=1)),
                    raises(ApiError, pattern="Context timeout"))



import os
import sys

from setuptools import Extension, setup
from setuptools.command.build_ext import build_ext as build_ext_orig


if sys.version_info < (3, 6):
    raise RuntimeError('Require Python 3.6 or greater')

PACKAGE_NAME = 'pyreindexer'


def _c2(*names):
    return ' :: '.join(names)


class CMakeExtension(Extension):
    def __init__(self, name):
        super().__init__(name, sources=[])


class BuildExt(build_ext_orig):
    def run(self):
        for ext in self.extensions:
            self.build_cmake(ext)

    def build_cmake(self, ext):
        cwd = os.path.abspath('')

        build_temp = os.path.abspath(self.build_temp)
        if not os.path.exists(build_temp):
            os.makedirs(build_temp)

        extension_dir = os.path.abspath(self.get_ext_fullpath(ext.name))
        if not os.path.exists(extension_dir):
            os.makedirs(extension_dir)

        os.chdir(build_temp)

        lib_dir = os.path.join(extension_dir, '..')
        source_dir = os.path.join(cwd, PACKAGE_NAME)

        self.spawn(['cmake', source_dir,
                    '-DCMAKE_BUILD_TYPE=RelWithDebInfo',
                    '-DCMAKE_CXX_STANDARD=17',
                    '-DCMAKE_LIBRARY_OUTPUT_DIRECTORY=' + lib_dir,
                    '-DCMAKE_OSX_DEPLOYMENT_TARGET=11'])
        if not self.dry_run:
            self.spawn(['cmake', '--build', '.'])

        os.chdir(cwd)


setup(name=PACKAGE_NAME,
      version='0.3.4',
      description='A connector that allows to interact with Reindexer',
      author='Igor Tulmentyev',
      maintainer='Reindexer Team',
      maintainer_email='contactus@reindexer.io',
      url='https://github.com/Restream/reindexer-py',
      project_urls={
          'Documentation': 'https://reindexer.io/',
          'Releases': 'https://github.com/Restream/reindexer-py/releases',
          'Tracker': 'https://github.com/Restream/reindexer-py/issues',
          'Telegram chat': 'https://t.me/reindexer',
      },
      long_description=open("README.md", encoding="utf-8").read(),
      long_description_content_type="text/markdown",
      license='Apache License 2.0',
      packages=[PACKAGE_NAME],
      ext_modules=[CMakeExtension('rawpyreindexer')],
      cmdclass={'build_ext': BuildExt},
      keywords=["reindexer", "in-memory-database", "database", "python", "connector"],
      package_data={'pyreindexer': [
          'CMakeLists.txt',
          'lib/**/*.h',
          'lib/**/*.cc',
          'example/main.py',
          'tests/**/*.py'
      ]},
      python_requires=">=3.6",
      test_suite='tests',
      install_requires=['envoy==0.0.3', 'delegator==0.0.3', 'pyhamcrest==2.0.2', 'pytest==6.2.5'],
      classifiers=[
          _c2('Development Status', '3 - Alpha'),
          _c2('Environment', 'Console'),
          _c2('Intended Audience', 'End Users/Desktop'),
          _c2('Intended Audience', 'Developers'),
          _c2('License', 'OSI Approved', 'Apache Software License'),
          _c2('Natural Language', 'Russian'),
          _c2('Operating System', 'MacOS'),
          _c2('Operating System', 'POSIX', 'Linux'),
          _c2('Programming Language', 'Python'),
          _c2('Programming Language', 'Python', '3.6'),
          _c2('Programming Language', 'Python', '3.7'),
          _c2('Programming Language', 'Python', '3.8'),
          _c2('Programming Language', 'Python', '3.9'),
          _c2('Programming Language', 'Python', '3.10'),
          _c2('Programming Language', 'Python', '3.11'),
          _c2('Programming Language', 'Python', '3.12'),
          _c2('Programming Language', 'Python', 'Implementation'),
          _c2('Programming Language', 'Python', 'Implementation', 'CPython'),
          _c2('Programming Language', 'Python', 'Implementation', 'PyPy'),
          _c2('Topic', 'Database'),
          _c2('Topic', 'Database', 'Database Engines/Servers'),
          _c2('Topic', 'Software Development'),
          _c2('Topic', 'Software Development', 'Libraries'),
          _c2('Topic', 'Software Development', 'Libraries', 'Python Modules'),
          _c2('Topic', 'Software Development', 'Libraries', 'Application Frameworks'),
      ],
      platforms=['ALT Linux', 'RED OS', 'Astra Linux'],
      )
